@@ -4,28 +4,44 @@ import * as React from 'react';
 import { useAsync } from 'react-use';
 
 import {
+  type AppPluginConfig,
   type PluginExtensionEventHelpers,
   type PluginExtensionOpenModalOptions,
   isDateTime,
   dateTime,
   PluginContextProvider,
-  PluginExtensionLink,
-  PanelMenuItem,
-  PluginExtensionAddedLinkConfig,
+  type PluginExtensionLink,
+  type PanelMenuItem,
+  type PluginExtensionAddedLinkConfig,
   urlUtil,
-  PluginExtensionPoints,
-  ExtensionInfo,
 } from '@grafana/data';
-import { reportInteraction, config, AppPluginConfig } from '@grafana/runtime';
+import { reportInteraction, config } from '@grafana/runtime';
+import { getAppPluginMetas } from '@grafana/runtime/internal';
 import { Modal } from '@grafana/ui';
-import appEvents from 'app/core/app_events';
+import { appEvents } from 'app/core/app_events';
 import { getPluginSettings } from 'app/features/plugins/pluginSettings';
-import { CloseExtensionSidebarEvent, OpenExtensionSidebarEvent, ShowModalReactEvent } from 'app/types/events';
+import {
+  CloseExtensionSidebarEvent,
+  OpenExtensionSidebarEvent,
+  ShowModalReactEvent,
+  ToggleExtensionSidebarEvent,
+} from 'app/types/events';
+
+import { RestrictedGrafanaApisProvider } from '../components/restrictedGrafanaApis/RestrictedGrafanaApisProvider';
 
 import { ExtensionErrorBoundary } from './ExtensionErrorBoundary';
+import {
+  getAppPluginConfigsSync,
+  getAppPluginsToAwaitSync,
+  getAppPluginsToPreloadSync,
+  getExposedComponentPluginDependenciesSync,
+  getExtensionPointPluginDependenciesSync,
+  getExtensionPointPluginMetaSync,
+  type ExtensionPointPluginMeta,
+} from './appUtils';
 import { ExtensionsLog, log as baseLog } from './logs/log';
 import { AddedLinkRegistryItem } from './registry/AddedLinksRegistry';
-import { assertIsNotPromise, assertLinkPathIsValid, assertStringProps, isPromise } from './validators';
+import { assertIsNotPromise, assertStringProps, isPromise } from './validators';
 
 export function handleErrorsInFn(fn: Function, errorMessagePrefix = '') {
   return (...args: unknown[]) => {
@@ -98,7 +114,11 @@ export const wrapWithPluginContext = <T,>({
     return (
       <PluginContextProvider meta={pluginMeta}>
         <ExtensionErrorBoundary pluginId={pluginId} extensionTitle={extensionTitle} log={log}>
-          <Component {...writableProxy(props, { log, source: 'extension', pluginId })} />
+          <RestrictedGrafanaApisProvider pluginId={pluginId}>
+            <Component
+              {...writableProxy(props, { log, source: 'extension', pluginId, pluginVersion: pluginMeta.info?.version })}
+            />
+          </RestrictedGrafanaApisProvider>
         </ExtensionErrorBoundary>
       </PluginContextProvider>
     );
@@ -408,6 +428,8 @@ export function createExtensionSubMenu(extensions: PluginExtensionLink[]): Panel
         text: truncateTitle(extension.title, 25),
         href: extension.path,
         onClick: extension.onClick,
+        iconClassName: extension.icon,
+        target: extension.openInNewTab ? '_blank' : undefined,
       });
       continue;
     }
@@ -420,6 +442,8 @@ export function createExtensionSubMenu(extensions: PluginExtensionLink[]): Panel
       text: truncateTitle(extension.title, 25),
       href: extension.path,
       onClick: extension.onClick,
+      iconClassName: extension.icon,
+      target: extension.openInNewTab ? '_blank' : undefined,
     });
   }
 
@@ -461,12 +485,14 @@ export function getLinkExtensionOverrides(
       return undefined;
     }
 
+    // Only allowing to override the following properties
     let {
       title = config.title,
       description = config.description,
       path = config.path,
       icon = config.icon,
       category = config.category,
+      openInNewTab = config.openInNewTab,
       ...rest
     } = overrides;
 
@@ -475,7 +501,6 @@ export function getLinkExtensionOverrides(
       `The configure() function for "${config.title}" returned a promise, skipping updates.`
     );
 
-    path && assertLinkPathIsValid(pluginId, path);
     assertStringProps({ title, description }, ['title', 'description']);
 
     if (Object.keys(rest).length > 0) {
@@ -492,6 +517,7 @@ export function getLinkExtensionOverrides(
       path,
       icon,
       category,
+      openInNewTab,
     };
   } catch (error) {
     if (error instanceof Error) {
@@ -531,6 +557,7 @@ export function getLinkExtensionOnClick(
 
       const helpers: PluginExtensionEventHelpers = {
         context,
+        extensionPointId,
         openModal: createOpenModalFunction(config),
         openSidebar: (componentTitle, context) => {
           appEvents.publish(
@@ -544,6 +571,15 @@ export function getLinkExtensionOnClick(
         closeSidebar: () => {
           appEvents.publish(new CloseExtensionSidebarEvent());
         },
+        toggleSidebar: (componentTitle, context) => {
+          appEvents.publish(
+            new ToggleExtensionSidebarEvent({
+              props: context,
+              pluginId,
+              componentTitle,
+            })
+          );
+        },
       };
 
       log.debug(`onClick '${config.title}' at '${extensionPointId}'`);
@@ -584,124 +620,63 @@ export function getLinkExtensionPathWithTracking(pluginId: string, path: string,
 // Can be set with the `GF_DEFAULT_APP_MODE` environment variable
 export const isGrafanaDevMode = () => config.buildInfo.env === 'development';
 
-export const getAppPluginConfigs = (pluginIds: string[] = []) =>
-  Object.values(config.apps).filter((app) => pluginIds.includes(app.id));
+/**
+ * Returns a list of app plugin configs that match the given plugin ids.
+ * @param pluginIds - The list of plugin ids to filter by.
+ * @returns A list of app plugin configs that match the given plugin ids.
+ */
+export async function getAppPluginConfigs(pluginIds: string[] = []): Promise<AppPluginConfig[]> {
+  const apps = await getAppPluginMetas();
+  return getAppPluginConfigsSync(pluginIds, apps);
+}
 
-export const getAppPluginIdFromExposedComponentId = (exposedComponentId: string) => {
-  return exposedComponentId.split('/')[0];
-};
-
-// Returns a list of app plugin ids that are registering extensions to this extension point.
-// (These plugins are necessary to be loaded to use the extension point.)
-// (The function also returns the plugin ids that the plugins - that extend the extension point - depend on.)
-export const getExtensionPointPluginDependencies = (extensionPointId: string): string[] => {
-  return Object.values(config.apps)
-    .filter(
-      (app) =>
-        app.extensions.addedLinks.some((link) => link.targets.includes(extensionPointId)) ||
-        app.extensions.addedComponents.some((component) => component.targets.includes(extensionPointId))
-    )
-    .map((app) => app.id)
-    .reduce((acc: string[], id: string) => {
-      return [...acc, id, ...getAppPluginDependencies(id)];
-    }, []);
-};
-
-export type ExtensionPointPluginMeta = Map<
-  string,
-  {
-    readonly addedComponents: ExtensionInfo[];
-    readonly addedLinks: ExtensionInfo[];
-  }
->;
+/**
+ * Returns a list of app plugin ids that are registering extensions to this extension point.
+ * (These plugins are necessary to be loaded to use the extension point.)
+ * (The function also returns the plugin ids that the plugins - that extend the extension point - depend on.)
+ * @param extensionPointId - The id of the extension point.
+ * @returns A list of app plugin ids that are registering extensions to this extension point.
+ */
+export async function getExtensionPointPluginDependencies(extensionPointId: string): Promise<string[]> {
+  const apps = await getAppPluginMetas();
+  return getExtensionPointPluginDependenciesSync(extensionPointId, apps);
+}
 
 /**
  * Returns a map of plugin ids and their addedComponents and addedLinks to the extension point.
  * @param extensionPointId - The id of the extension point.
  * @returns A map of plugin ids and their addedComponents and addedLinks to the extension point.
  */
-export const getExtensionPointPluginMeta = (extensionPointId: string): ExtensionPointPluginMeta => {
-  return new Map(
-    getExtensionPointPluginDependencies(extensionPointId)
-      .map((pluginId) => {
-        const app = config.apps[pluginId];
-        // if the plugin does not exist or does not expose any components or links to the extension point, return undefined
-        if (
-          !app ||
-          (!app.extensions.addedComponents.some((component) => component.targets.includes(extensionPointId)) &&
-            !app.extensions.addedLinks.some((link) => link.targets.includes(extensionPointId)))
-        ) {
-          return undefined;
-        }
-        return [
-          pluginId,
-          {
-            addedComponents: app.extensions.addedComponents.filter((component) =>
-              component.targets.includes(extensionPointId)
-            ),
-            addedLinks: app.extensions.addedLinks.filter((link) => link.targets.includes(extensionPointId)),
-          },
-        ] as const;
-      })
-      .filter((c): c is NonNullable<typeof c> => c !== undefined)
-  );
-};
+export async function getExtensionPointPluginMeta(extensionPointId: string): Promise<ExtensionPointPluginMeta> {
+  const apps = await getAppPluginMetas();
+  return getExtensionPointPluginMetaSync(extensionPointId, apps);
+}
 
-// Returns a list of app plugin ids that are necessary to be loaded to use the exposed component.
-// (It is first the plugin that exposes the component, and then the ones that it depends on.)
-export const getExposedComponentPluginDependencies = (exposedComponentId: string) => {
-  const pluginId = getAppPluginIdFromExposedComponentId(exposedComponentId);
+/**
+ * Returns a list of app plugin ids that are necessary to be loaded to use the exposed component.
+ * (It is first the plugin that exposes the component, and then the ones that it depends on.)
+ * @param exposedComponentId - The id of the exposed component.
+ * @returns A list of app plugin ids that are necessary to be loaded to use the exposed component.
+ */
+export async function getExposedComponentPluginDependencies(exposedComponentId: string): Promise<string[]> {
+  const apps = await getAppPluginMetas();
+  return getExposedComponentPluginDependenciesSync(exposedComponentId, apps);
+}
 
-  return [pluginId].reduce((acc: string[], pluginId: string) => {
-    return [...acc, pluginId, ...getAppPluginDependencies(pluginId)];
-  }, []);
-};
+/**
+ * Returns a list of app plugins that has to be loaded before core Grafana could finish the initialization.
+ * @returns A list of app plugins that has to be loaded before core Grafana could finish the initialization.
+ */
+export async function getAppPluginsToAwait(): Promise<AppPluginConfig[]> {
+  const apps = await getAppPluginMetas();
+  return getAppPluginsToAwaitSync(apps);
+}
 
-// Returns a list of app plugin ids that are necessary to be loaded, based on the `dependencies.extensions`
-// metadata field. (For example the plugins that expose components that the app depends on.)
-// Heads up! This is a recursive function.
-export const getAppPluginDependencies = (pluginId: string, visited: string[] = []): string[] => {
-  if (!config.apps[pluginId]) {
-    return [];
-  }
-
-  // Prevent infinite recursion (it would happen if there is a circular dependency between app plugins)
-  if (visited.includes(pluginId)) {
-    return [];
-  }
-
-  const pluginIdDependencies = config.apps[pluginId].dependencies.extensions.exposedComponents.map(
-    getAppPluginIdFromExposedComponentId
-  );
-
-  return (
-    pluginIdDependencies
-      .reduce((acc, _pluginId) => {
-        return [...acc, ...getAppPluginDependencies(_pluginId, [...visited, pluginId])];
-      }, pluginIdDependencies)
-      // We don't want the plugin to "depend on itself"
-      .filter((id) => id !== pluginId)
-  );
-};
-
-// Returns a list of app plugins that has to be loaded before core Grafana could finish the initialization.
-export const getAppPluginsToAwait = () => {
-  const pluginIds = [
-    // The "cloud-home-app" is registering banners once it's loaded, and this can cause a rerender in the AppChrome if it's loaded after the Grafana app init.
-    'cloud-home-app',
-  ];
-
-  return Object.values(config.apps).filter((app) => pluginIds.includes(app.id));
-};
-
-// Returns a list of app plugins that has to be preloaded in parallel with the core Grafana initialization.
-export const getAppPluginsToPreload = () => {
-  // The DashboardPanelMenu extension point is using the `getPluginExtensions()` API in scenes at the moment, which means that it cannot yet benefit from dynamic plugin loading.
-  const dashboardPanelMenuPluginIds = getExtensionPointPluginDependencies(PluginExtensionPoints.DashboardPanelMenu);
-  const awaitedPluginIds = getAppPluginsToAwait().map((app) => app.id);
-  const isNotAwaited = (app: AppPluginConfig) => !awaitedPluginIds.includes(app.id);
-
-  return Object.values(config.apps).filter((app) => {
-    return isNotAwaited(app) && (app.preload || dashboardPanelMenuPluginIds.includes(app.id));
-  });
-};
+/**
+ * Returns a list of app plugins that has to be preloaded in parallel with the core Grafana initialization.
+ * @returns An array of app plugin configs that has to be preloaded in parallel with the core Grafana initialization.
+ */
+export async function getAppPluginsToPreload(): Promise<AppPluginConfig[]> {
+  const apps = await getAppPluginMetas();
+  return getAppPluginsToPreloadSync(apps);
+}
